unit World;

interface

uses
  Dialogs, Graphics, Tiles, SysUtils;

type
  TLevelData = record
    WidthTiles: Integer;
    HeightTiles: Integer;
    TLRFile: String[255];
    //Bice ovde jos stvari, npr broj jabuka na nivou...
  end;

type
  TWorld = class
  private
    FTiles: TTiles;
    FWidthTiles, FHeightTiles: Integer;
    FLevelMap: array of array of Byte;
  public
    constructor Create;
    destructor Destroy; override;
    function LoadMap(FileName: String): Boolean;
    function IsAvailablePos(X, Y: Integer): Boolean;
  end;

implementation

{ TWorld }

constructor TWorld.Create;
begin
  FWidthTiles := 0;
  FHeightTiles := 0;
  FTiles := TTiles.Create;
end;

destructor TWorld.Destroy;
begin
  SetLength(FLevelMap, 0, 0);
  FLevelMap := nil;
  inherited Destroy;
end;

function TWorld.IsAvailablePos(X, Y: Integer): Boolean;
{var
  B: byte;
  LvX, LvY: Integer;
  MapX, MapY: Integer;}
begin
  {
  LvX := X div TileWidth;
  LvY := Y div TileHeight;
  MapX := X mod TileWidth;
  MapY := Y mod TileHeight;
  B := FLevelMap[LvX, LvY];
  B := FTiles.GetFreemap(B)[MapX, MapY] ;
  Result := B = 0;
  }
  Result := FTiles.GetFreemap(FLevelMap[X div TileWidth, Y div TileHeight])[X mod TileWidth, Y mod TileHeight] <= MapCodeWater;
end;

function TWorld.LoadMap(FileName: String): Boolean;
var
  I, J: Integer;
  FLD: file of TLevelData;
  FLM: file of Byte;
  LD: TLevelData;
  LM: Byte;  
begin
  if not FileExists(FileName + '.lvd') then
  begin
    ShowMessage('Unable to load resource file (' + Filename + '.lvd)');
    Result := False;
    Exit;
  end;
  if not FileExists(FileName + '.lvm') then
  begin
    ShowMessage('Unable to load resource file (' + Filename + '.lvm)');
    Result := False;
    Exit;
  end;
  try
    AssignFile(FLD, FileName + '.lvd');
    Reset(FLD);
    Read(FLD, LD);
    FWidthTiles := LD.WidthTiles;
    FHeightTiles := LD.HeightTiles;
    if not FTiles.LoadTiles(ExtractFilePath(FileName + '.xxx') + Trim(LD.TLRFile)) then
    begin
      Result := False;
      Exit;
    end;
    AssignFile(FLM, FileName + '.lvm');
    Reset(FLM);
    SetLength(FLevelMap, LD.WidthTiles, LD.HeightTiles);
    for J := FHeightTiles - 1 downto 0 do
    begin
      for I := 0 to FWidthTiles - 1 do
      begin
        Read(FLM, LM);
        LM := LM - Ord('0'); //Bice izbaceno, za sada potrebno da bi textedtorom mogao da se pravi nivo
        FLevelMap[I, J] := LM;
      end;
      Read(FLM, LM); // Chr#13 novi red u LVM fajlu
      Read(FLM, LM); // Chr#10
    end;
    Result := True;
  except
    ShowMessage('Error during levelmap loading!');
    Result := False;
  end;
end;

end.
